package de.dassit.dassadmin;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

import de.dassit.ldap.Connection;
import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;
import de.dassit.ldap.User;

public class DassUser extends SmbUser {
	private static final Logger log = Logger.getLogger(DassUser.class
			.getName());
	
	private static final String OX_ACCESS_ATTRIBUTE = "carLicense";

	static String templateContainer = "ou=dassadmin";

	static String primaryMailAttribute = "mail";

	static String mailAliasesAttribute = "suseMailAcceptAddress";

	static String imapQuotaAttribute = "imapQuota";

	static String enabledAttribute;

	static String enabledValue;

	static String disabledValue;

	static boolean enabledDefault = true;

	static String extraObjectClasses;

	public DassUser() {
		super();
	}

	public DassUser(SearchResult arg0) {
		super(arg0);
	}

	public DassUser(String arg0, int arg1, int arg2, String arg3, String arg4,
			String arg5) {
		super(arg0, arg1, arg2, arg3, arg4, arg5);
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<DassUser>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassUser(sr));
			}
		};
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "sambaSamAccount",
				"inetOrgPerson", "posixAccount" });
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public void setExtraObjectClasses(String csv) {
		extraObjectClasses = csv;
	}

	public void setPrimaryMailAttribute(String m) {
		primaryMailAttribute = m;
	}

	public void setMailAliasesAttribute(String m) {
		mailAliasesAttribute = m;
	}

	public void setEnabledAttribute(String s) {
		enabledAttribute = s;
	}

	public void setEnabledValue(String s) {
		enabledValue = s;
	}

	public void setDisabledValue(String s) {
		disabledValue = s;
	}

	public void setEnabledDefault(boolean b) {
		enabledDefault = b;
	}

	public void setDisabledNumber(int n) {
		setDisabledValue(Integer.toString(n));
	}

	public void setEnabledNumber(int n) {
		setEnabledValue(Integer.toString(n));
	}

	public void setImapQuotaAttribute(String a) {
		imapQuotaAttribute = a;
	}

	public void setDefaultContainer(String ou) {
		super.setDefaultContainer(ou);
		User factory = new User();
		factory.setDefaultContainer(ou);
	}

	public void setTemplateContainer(String ou) {
		templateContainer = ou;
	}

	public void setSurname(String value) {
		put("sn", value);
	}

	public void setImapQuota(String value) {
		put(imapQuotaAttribute, value);
	}

	public String getImapQuota() {
		return getAttribute(imapQuotaAttribute);
	}

	public void setEnabled(boolean b) {
		if (enabledAttribute == null)
			return;

		if (b) {
			put(enabledAttribute, enabledValue);
		} else {
			put(enabledAttribute, disabledValue);
		}
	}

	public boolean getEnabled() {
		if (enabledAttribute == null)
			return true;

		String value = getAttribute(enabledAttribute);

		if (value == null)
			value = "";

		if (disabledValue != null && value.equalsIgnoreCase(disabledValue))
			return false;
		if (enabledValue != null && value.equalsIgnoreCase(enabledValue))
			return true;

		return enabledDefault;
	}

	public void setPrimaryMailAddress(String s) {
		put(primaryMailAttribute, s);
	}

	public String getPrimaryMailAddress() {
		return getAttribute(primaryMailAttribute);
	}

	public ArrayList<String> getMailAliases() {
		return getAttributeMultiValue(mailAliasesAttribute);
	}

	public void addMailAlias(String s) {
		appendAttributeValue(mailAliasesAttribute, s);
	}

	public void removeMailAlias(String s) {
		removeAttributeValue(mailAliasesAttribute, s);
	}

	public String getGroupName() {
		return DassGroup.fetchById(getGid()).getName();
	}

	public String getOxAccess() {
		return getAttribute(OX_ACCESS_ATTRIBUTE);
	}

	public void setOxAccess(String s) {
		put(OX_ACCESS_ATTRIBUTE, s);
	}

	public List<DassUser> getAll() {
		return searchDassUsers("objectClass=posixAccount", defaultContainer);
	}

	public static User getTemplateUser() {
		User u = searchDassUsers("uid=templateUser", templateContainer).get(0);
		u.setContainer(getDefaultContainer(DassUser.class.getName()));
		u.saveAsNew();
		return u;
	}

	public static List<DassUser> searchDassUsers(String filter, String container) {

		EntryList<DassUser> entryList = new EntryList<DassUser>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassUser(sr));
			}
		};

		Connection.getDefaultConnection().search(filter, container, entryList);
		List<DassUser> result = entryList.getEntries();

		Collections.sort(result, new Comparator<DassUser>() {
			public int compare(DassUser arg0, DassUser arg1) {
				String sn0 = arg0.getSurname();
				String sn1 = arg1.getSurname();

				if (sn0 == sn1 && sn0 == null)
					return 0;
				if (sn0 == null)
					return -1;
				if (sn1 == null)
					return 1;

				int r1 = sn0.compareToIgnoreCase(sn1);
				if (r1 != 0)
					return r1;

				String gn0 = arg0.getGivenName();
				String gn1 = arg1.getGivenName();

				if (gn0 == gn1 && gn0 == null)
					return 0;

				if (gn0 == null)
					return -1;
				if (gn1 == null)
					return 1;

				int r2 = gn0.compareToIgnoreCase(gn1);
				return r2;
			}
		});
		return entryList.getEntries();
	}

	public static synchronized int getNextUid() throws NamingException {
		List<DassUser> result = DassUser.searchDassUsers("uid=templateUser",
				templateContainer);
		User template = result.get(0);

		int nextUid = template.getUid();

		log.info("next uidNumber " + nextUid);
		User factory = new User();
		while (factory.search("uidNumber=" + nextUid,
				getDefaultContainer(factory.getClass().getName())).size() != 0) {
			nextUid++;
			log.info("trying uidNumber " + nextUid);
		}
		log.info(nextUid + " ok");
		template.setUid(nextUid + 1);
		template.save();
		return nextUid;
	}

	public List<DassGroup> getDassGroups() {
		return DassGroup.searchDassByMemberName(getName());
	}

	public void removeFromDassGroups() throws NamingException {
		for (DassGroup g : getDassGroups()) {
			g.removeMemberName(getName());
			g.save();
		}
	}

	@Override
	public void delete() throws NamingException {
		removeFromDassGroups();
		super.delete();
	}

	@Override
	public void rename(String newName) throws NamingException {
		List<DassGroup> groups = getDassGroups();
		String oldName = getName();
		String oldDn = getDN();

		rename(Connection.getDefaultConnection(), newName);

		for (DassGroup g : groups) {
			g.removeMemberName(oldName);
			g.addMemberName(newName);
			g.save();
		}

		List<DassUser> subordinates = searchDassUsers("manager=" + oldDn,
				container);

		for (User u : subordinates) {
			log.info(u.getDN());
			u.put("manager", getDN());
			u.save();
		}

	}
}