package de.dassit.dassadmin;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Logger;

public class QueueStatusTool {
	private static final Logger log = Logger.getLogger(QueueStatusTool.class
			.getName());

	private int port = 8098;

	private String host = "127.0.0.1";

	private int errorCount = -2;
	private int jobCount = -2;
	private boolean fetched = false;

	/**
	 * @param args
	 * @throws IOException
	 * @throws UnknownHostException
	 */

	public void setPort(int port) {
		this.port = port;
	}

	public void fetchData() throws UnknownHostException {
		Socket sock = null;
		String line = null;
		try {
			sock = new Socket(host, port);

			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new DataInputStream(sock.getInputStream())));
			line = reader.readLine();
			reader.close();
		} catch (IOException ex) {
			return;
		} finally {
			if (sock != null)
				try {
					sock.close();
				} catch (IOException e) {
					// ignore that
				}
		}

		if (line == null)
			return;

		fetched = true;

		String[] parts = line.split(" ");

		if (parts.length < 2) {
			jobCount = 0;
			errorCount = 1;
			return;
		}

		jobCount = Integer.parseInt(parts[0]);
		errorCount = Integer.parseInt(parts[1]);
	}

	public int getErrorCount() throws UnknownHostException, IOException {
		if (!fetched)
			fetchData();
		return errorCount;
	}

	public int getJobCount() throws UnknownHostException, IOException {
		if (!fetched)
			fetchData();
		return jobCount;
	}

	public static void main(String[] args) throws UnknownHostException,
			IOException {
		QueueStatusTool statusTool = new QueueStatusTool();

		log.info("jobs processed: " + statusTool.getJobCount());
	}

}
