package de.dassit.ldap;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;

public class QueueLogger implements ChangeLogger {

	static File lockfile = new File("/var/spool/dassadmin/lock");

	static File lockDirectory = new File("/var/spool/dassadmin");

	RandomAccessFile lock;

	protected synchronized void queue(String message) {
		try {
			openAndLock();

			int jobNumber = readAndIncrJobNumber();

			FileWriter jobFile = new FileWriter(new File(lockDirectory, Integer
					.toString(jobNumber)));
			jobFile.write(message);
			jobFile.close();
			
		} catch (IOException ioex) {
			// log and continue
			ioex.printStackTrace();
		} finally {
			try {
				releaseLock();
			}
			catch(IOException ioex2) {
				ioex2.printStackTrace();
			}
		}
	}

	private void releaseLock() throws IOException {
		lock.close();
	}

	private void openAndLock() throws IOException {
		lock = new RandomAccessFile(lockfile, "rw");
		FileChannel fd = lock.getChannel();
		fd.lock();
	}

	private int readAndIncrJobNumber() throws IOException {
		int jobNumber = Integer.parseInt(lock.readLine()) + 1;
		lock.seek(0);
		lock.writeBytes(Integer.toString(jobNumber));
		return jobNumber;
	}

	protected String dequeue(String message) {
		return "NOT IMPLEMENTED";
	}

	public void create(String dn) {
		queue("C\n" + dn + "\n");
	}

	public void delete(String dn) {
		queue("D\n" + dn + "\n");

	}

	public void rename(String olddn, String newdn) {
		queue("R\n" + newdn + "\n" + olddn + "\n");
	}

	public void update(String dn) {
		queue("U\n" + dn + "\n");
	}

	public static void main(String[]  args) {
		QueueLogger q=new QueueLogger();
		q.rename("old", "new");
	}
}
